from django import forms
from django.core.validators import RegexValidator
from django.core.exceptions import ValidationError
from datetime import date
import re

class UserForm(forms.Form):
    nama = forms.CharField(
        max_length=255,
        label="Nama",
        validators=[
            RegexValidator(
                regex=r'^[A-Za-z0-9._-]+$',
                message='Nama hanya boleh berisi huruf, angka, dan karakter . _ -'
            )
        ]
    )
    password = forms.CharField(
        max_length=255,
        widget=forms.PasswordInput,
        label="Password"
    )
    tanggal_lahir = forms.DateField(
        widget=forms.DateInput(attrs={'type': 'date'}),
        label="Tanggal Lahir"
    )
    nomor_hp = forms.CharField(max_length=15, label="Nomor HP")
    email = forms.EmailField(max_length=255, label="Email")
    url_blog = forms.URLField(max_length=255, label="URL Blog")
    deskripsi_diri = forms.CharField(
        widget=forms.Textarea,
        min_length=5,
        max_length=1000,
        label="Deskripsi Diri"
    )
    
    STATUS_PEKERJAAN_CHOICES = [
        ('Karyawan', 'Karyawan'),
        ('PNS', 'PNS'),
        ('Mahasiswa', 'Mahasiswa'),
        ('Dosen', 'Dosen'),
    ]
    status_pekerjaan = forms.ChoiceField(
        choices=STATUS_PEKERJAAN_CHOICES,
        label="Status Pekerjaan",
        required=True
    )
    kode_mata_uang = forms.CharField(
        label="Kode Mata Uang",
        max_length=3,
        validators=[
            RegexValidator(
                regex=r'^[A-Za-z]{3}$',
                message="Harus 3 huruf (USD, IDR, dsb.)"
            )
        ]
    )

    def clean_password(self):
        pwd = self.cleaned_data['password']
        if len(pwd) < 8:
            raise ValidationError("Password minimal 8 karakter.")
        if not re.search(r'[A-Za-z]', pwd):
            raise ValidationError("Minimal 1 huruf.")
        if not re.search(r'\d', pwd):
            raise ValidationError("Minimal 1 angka.")
        if not re.search(r'[^A-Za-z0-9]', pwd):
            raise ValidationError("Minimal 1 karakter spesial.")
        return pwd
    
    def clean_tanggal_lahir(self):
        dob = self.cleaned_data['tanggal_lahir']
        today = date.today()
        age = today.year - dob.year - ((today.month, today.day) < (dob.month, dob.day))
        if age < 12:
            raise ValidationError("Umur minimal 12 tahun.")
        return dob
    
    def clean_nomor_hp(self):
        nomor = self.cleaned_data['nomor_hp']
        sanitized = nomor.replace(" ", "").replace("+", "").replace("-", "")
        if not sanitized.isdigit():
            raise ValidationError("Nomor HP harus berisi angka saja.")
        if not (8 <= len(sanitized) <= 15):
            raise ValidationError("Nomor HP harus 8 - 15 digit.")
        return sanitized


class LoginForm(forms.Form):
    username = forms.CharField(
        max_length=150,
        label='Username',
        validators=[
            RegexValidator(
                regex=r'^[A-Za-z0-9._-]+$',
                message='Username hanya boleh huruf, angka, . _ -'
            )
        ]
    )
    password = forms.CharField(
        max_length=255,
        widget=forms.PasswordInput,
        label='Password'
    )

    def clean_username(self):
        """Opsional: sanitasi atau validasi username lebih lanjut."""
        uname = self.cleaned_data.get('username', '').strip()
        # contoh: hapus spasi di awal/akhir
        # bisa tambahkan logika jika diinginkan
        return uname

    def clean_password(self):
        """Opsional: cek minimal length, dsb. 
           Namun ingat: di login, kita biasanya tidak butuh detail validasi 
           sebanyak di register."""
        pwd = self.cleaned_data['password']
        if len(pwd) < 8:
            raise ValidationError("Password minimal 8 karakter.")
        # dll. (jika diinginkan)
        return pwd
