from django.db import models
from django.contrib.auth.models import AbstractUser
from django.core.validators import URLValidator, RegexValidator
from django.core.exceptions import ValidationError
from datetime import date

STATUS_PEKERJAAN_CHOICES = [
    ('Karyawan', 'Karyawan'),
    ('PNS', 'PNS'),
    ('Mahasiswa', 'Mahasiswa'),
    ('Dosen', 'Dosen'),
]

class MyUser(AbstractUser):
    """
    Custom user model yang menurunkan AbstractUser.
    Django akan meng-hash password secara otomatis bila kita gunakan
    form/perintah set_password().
    """
    # Field tambahan sesuai requirement Week 6/7
    tanggal_lahir = models.DateField(blank=True, null=True)
    nomor_hp = models.CharField(max_length=15, blank=True, null=True)
    url_blog = models.CharField(max_length=255, blank=True, null=True, validators=[URLValidator()])
    deskripsi_diri = models.TextField(blank=True, null=True)
    status_pekerjaan = models.CharField(max_length=50, choices=STATUS_PEKERJAAN_CHOICES, blank=True, null=True)
    kode_mata_uang = models.CharField(max_length=3, blank=True, null=True,
                                      validators=[RegexValidator(regex=r'^[A-Za-z]{3}$',
                                      message='Contoh: USD, IDR, dsb.')])

    # Opsional: jika ingin menampung "Nama" sama persis
    # tapi kita juga bisa memetakan 'nama' -> 'username' di views
    nama_asli = models.CharField(max_length=255, blank=True, null=True,
        help_text="Field opsional untuk menyimpan 'nama' dari forms.py")

    def clean(self):
        # Contoh validasi umur, sanitasi nomor HP di level model
        if self.tanggal_lahir:
            today = date.today()
            age = today.year - self.tanggal_lahir.year - (
                (today.month, today.day) < (self.tanggal_lahir.month, self.tanggal_lahir.day)
            )
            if age < 12:
                raise ValidationError({'tanggal_lahir': 'Umur harus minimal 12 tahun.'})

        if self.nomor_hp:
            sanitized = self.nomor_hp.replace(" ", "").replace("+", "").replace("-", "")
            if not sanitized.isdigit():
                raise ValidationError({'nomor_hp': 'Nomor HP harus angka saja.'})
            if not (8 <= len(sanitized) <= 15):
                raise ValidationError({'nomor_hp': 'Nomor HP harus 8-15 digit.'})
            self.nomor_hp = sanitized
        
        super().clean()

    def __str__(self):
        return f"{self.username} - {self.email}"
